Index: wp-login.php
===================================================================
--- wp-login.php	(revision 6639)
+++ wp-login.php	(working copy)
@@ -1,34 +1,12 @@
 <?php
 require( dirname(__FILE__) . '/wp-config.php' );
 
-$action = $_REQUEST['action'];
-$errors = array();
-
-if ( isset($_GET['key']) )
-	$action = 'resetpass';
-
-nocache_headers();
-
-header('Content-Type: '.get_bloginfo('html_type').'; charset='.get_bloginfo('charset'));
-
-if ( defined('RELOCATE') ) { // Move flag is set
-	if ( isset( $_SERVER['PATH_INFO'] ) && ($_SERVER['PATH_INFO'] != $_SERVER['PHP_SELF']) )
-		$_SERVER['PHP_SELF'] = str_replace( $_SERVER['PATH_INFO'], '', $_SERVER['PHP_SELF'] );
-
-	$schema = ( isset($_SERVER['HTTPS']) && strtolower($_SERVER['HTTPS']) == 'on' ) ? 'https://' : 'http://';
-	if ( dirname($schema . $_SERVER['HTTP_HOST'] . $_SERVER['PHP_SELF']) != get_option('siteurl') )
-		update_option('siteurl', dirname($schema . $_SERVER['HTTP_HOST'] . $_SERVER['PHP_SELF']) );
-}
-
-//Set a cookie now to see if they are supported by the browser.
-setcookie(TEST_COOKIE, 'WP Cookie check', 0, COOKIEPATH, COOKIE_DOMAIN);
-if ( SITECOOKIEPATH != COOKIEPATH )
-	setcookie(TEST_COOKIE, 'WP Cookie check', 0, SITECOOKIEPATH, COOKIE_DOMAIN);
-
 // Rather than duplicating this HTML all over the place, we'll stick it in function
-function login_header($title = 'Login', $message = '') {
-	global $errors, $error;
+function login_header($title = 'Login', $message = '', $wp_error = '') {
+	global $error;
 
+	if ( empty($wp_error) )
+		$wp_error = new WP_Error();
 	?>
 <!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
 <html xmlns="http://www.w3.org/1999/xhtml" <?php language_attributes(); ?>>
@@ -50,30 +28,182 @@
 <?php
 	if ( !empty( $message ) ) echo apply_filters('login_message', $message) . "\n";
 
-	// Incase a plugin uses $error rather than the $errors array
+	// Incase a plugin uses $error rather than the $errors object
 	if ( !empty( $error ) ) {
-		$errors['error'] = $error;
+		$wp_error->add('error', $error);
 		unset($error);
 	}
 
-	if ( !empty( $errors ) ) {
-		if ( is_array( $errors ) ) {
-			$newerrors = "\n";
-			foreach ( $errors as $error ) $newerrors .= '	' . $error . "<br />\n";
-			$errors = $newerrors;
-		}
+	if ( $wp_error->get_error_code() ) {
+		$errors = "\n";
+		foreach ( $wp_error->get_error_messages() as $error )
+			$errors .= '	' . $error . "<br />\n";
 
 		echo '<div id="login_error">' . apply_filters('login_errors', $errors) . "</div>\n";
 	}
 } // End of login_header()
 
+function retrieve_password() {
+	global $wpdb;
+
+	$errors = new WP_Error();
+
+	if ( empty( $_POST['user_login'] ) && empty( $_POST['user_email'] ) )
+		$errors->add('empty_username', __('<strong>ERROR</strong>: Enter a username or e-mail address.'));
+
+	if ( strstr($_POST['user_login'], '@') ) {
+		$user_data = get_user_by_email(trim($_POST['user_login']));
+		if ( empty($user_data) )
+			$errors->add('invalid_email', __('<strong>ERROR</strong>: There is no user registered with that email address.'));
+	} else {
+		$login = trim($_POST['user_login']);
+		$user_data = get_userdatabylogin($login);
+	}
+
+	do_action('lostpassword_post');
+
+	if ( $errors->get_error_code() )
+		return $errors;
+	
+	if ( !$user_data ) {
+		$errors->add('invalidcombo', __('<strong>ERROR</strong>: Invalid username or e-mail.'));
+		return $errors;
+	}
+
+	// redefining user_login ensures we return the right case in the email
+	$user_login = $user_data->user_login;
+	$user_email = $user_data->user_email;
+
+	do_action('retreive_password', $user_login);  // Misspelled and deprecated
+	do_action('retrieve_password', $user_login);
+
+	// Generate something random for a key...
+	$key = wp_generate_password();
+	// Now insert the new md5 key into the db
+	$wpdb->query("UPDATE $wpdb->users SET user_activation_key = '$key' WHERE user_login = '$user_login'");
+	$message = __('Someone has asked to reset the password for the following site and username.') . "\r\n\r\n";
+	$message .= get_option('siteurl') . "\r\n\r\n";
+	$message .= sprintf(__('Username: %s'), $user_login) . "\r\n\r\n";
+	$message .= __('To reset your password visit the following address, otherwise just ignore this email and nothing will happen.') . "\r\n\r\n";
+	$message .= get_option('siteurl') . "/wp-login.php?action=rp&key=$key\r\n";
+
+	if ( !wp_mail($user_email, sprintf(__('[%s] Password Reset'), get_option('blogname')), $message) )
+		die('<p>' . __('The e-mail could not be sent.') . "<br />\n" . __('Possible reason: your host may have disabled the mail() function...') . '</p>');
+
+	return true;
+}
+
+function reset_password($key) {
+	global $wpdb;
+
+	$key = preg_replace('/[^a-z0-9]/i', '', $key);
+
+	if ( empty( $key ) )
+		return new WP_Error('invalid_key', __('Invalid key'));
+
+	$user = $wpdb->get_row("SELECT * FROM $wpdb->users WHERE user_activation_key = '$key'");
+	if ( empty( $user ) )
+		return new WP_Error('invalid_key', __('Invalid key'));
+
+	do_action('password_reset');
+
+	// Generate something random for a password...
+	$new_pass = wp_generate_password();
+	wp_set_password($new_pass, $user->ID);
+	$message  = sprintf(__('Username: %s'), $user->user_login) . "\r\n";
+	$message .= sprintf(__('Password: %s'), $new_pass) . "\r\n";
+	$message .= get_option('siteurl') . "/wp-login.php\r\n";
+
+	if (  !wp_mail($user->user_email, sprintf(__('[%s] Your new password'), get_option('blogname')), $message) )
+		die('<p>' . __('The e-mail could not be sent.') . "<br />\n" . __('Possible reason: your host may have disabled the mail() function...') . '</p>');
+
+	// send a copy of password change notification to the admin
+	// but check to see if it's the admin whose password we're changing, and skip this
+	if ( $user->user_email != get_option('admin_email') ) {
+		$message = sprintf(__('Password Lost and Changed for user: %s'), $user->user_login) . "\r\n";
+		wp_mail(get_option('admin_email'), sprintf(__('[%s] Password Lost/Changed'), get_option('blogname')), $message);
+	}
+
+	return true;
+}
+
+function register_new_user($user_login, $user_email) {
+	$errors = new WP_Error();
+
+	$user_login = sanitize_user( $user_login );
+	$user_email = apply_filters( 'user_registration_email', $user_email );
+
+	// Check the username
+	if ( $user_login == '' )
+		$errors->add('empty_username', __('<strong>ERROR</strong>: Please enter a username.'));
+	elseif ( !validate_username( $user_login ) ) {
+		$errors->add('invalid_username', __('<strong>ERROR</strong>: This username is invalid.  Please enter a valid username.'));
+		$user_login = '';
+	} elseif ( username_exists( $user_login ) )
+		$errors->add('username_exists', __('<strong>ERROR</strong>: This username is already registered, please choose another one.'));
+
+	// Check the e-mail address
+	if ($user_email == '') {
+		$errors->add('empty_email', __('<strong>ERROR</strong>: Please type your e-mail address.'));
+	} elseif ( !is_email( $user_email ) ) {
+		$errors->add('invalid_email', __('<strong>ERROR</strong>: The email address isn&#8217;t correct.'));
+		$user_email = '';
+	} elseif ( email_exists( $user_email ) )
+		$errors->add('email_exists', __('<strong>ERROR</strong>: This email is already registered, please choose another one.'));
+
+	do_action('register_post');
+
+	$errors = apply_filters( 'registration_errors', $errors );
+
+	if ( $errors->get_error_code() )
+		return $errors;
+
+	$user_pass = wp_generate_password();
+	$user_id = wp_create_user( $user_login, $user_pass, $user_email );
+	if ( !$user_id ) {
+		$errors->add('registerfail', sprintf(__('<strong>ERROR</strong>: Couldn&#8217;t register you... please contact the <a href="mailto:%s">webmaster</a> !'), get_option('admin_email')));
+		return $errors;
+	}
+
+	wp_new_user_notification($user_id, $user_pass);
+
+	return $user_id;
+}
+
+// 
+// Main
+//
+
+$action = $_REQUEST['action'];
+$errors = new WP_Error();
+
+if ( isset($_GET['key']) )
+	$action = 'resetpass';
+
+nocache_headers();
+
+header('Content-Type: '.get_bloginfo('html_type').'; charset='.get_bloginfo('charset'));
+
+if ( defined('RELOCATE') ) { // Move flag is set
+	if ( isset( $_SERVER['PATH_INFO'] ) && ($_SERVER['PATH_INFO'] != $_SERVER['PHP_SELF']) )
+		$_SERVER['PHP_SELF'] = str_replace( $_SERVER['PATH_INFO'], '', $_SERVER['PHP_SELF'] );
+
+	$schema = ( isset($_SERVER['HTTPS']) && strtolower($_SERVER['HTTPS']) == 'on' ) ? 'https://' : 'http://';
+	if ( dirname($schema . $_SERVER['HTTP_HOST'] . $_SERVER['PHP_SELF']) != get_option('siteurl') )
+		update_option('siteurl', dirname($schema . $_SERVER['HTTP_HOST'] . $_SERVER['PHP_SELF']) );
+}
+
+//Set a cookie now to see if they are supported by the browser.
+setcookie(TEST_COOKIE, 'WP Cookie check', 0, COOKIEPATH, COOKIE_DOMAIN);
+if ( SITECOOKIEPATH != COOKIEPATH )
+	setcookie(TEST_COOKIE, 'WP Cookie check', 0, SITECOOKIEPATH, COOKIE_DOMAIN);
+
 $http_post = ('POST' == $_SERVER['REQUEST_METHOD']);
 switch ($action) {
 
 case 'logout' :
 
-	wp_clearcookie();
-	do_action('wp_logout');
+	wp_logout();
 
 	$redirect_to = 'wp-login.php?loggedout=true';
 	if ( isset( $_REQUEST['redirect_to'] ) )
@@ -86,64 +216,25 @@
 
 case 'lostpassword' :
 case 'retrievepassword' :
-	$user_login = '';
-	$user_pass = '';
-
 	if ( $http_post ) {
-		if ( empty( $_POST['user_login'] ) )
-			$errors['user_login'] = __('<strong>ERROR</strong>: The username field is empty.');
-		if ( empty( $_POST['user_email'] ) )
-			$errors['user_email'] = __('<strong>ERROR</strong>: The e-mail field is empty.');
-
-		do_action('lostpassword_post');
-
-		if ( empty( $errors ) ) {
-			$user_data = get_userdatabylogin(trim($_POST['user_login']));
-			// redefining user_login ensures we return the right case in the email
-			$user_login = $user_data->user_login;
-			$user_email = $user_data->user_email;
-
-			if (!$user_email || $user_email != $_POST['user_email']) {
-				$errors['invalidcombo'] = __('<strong>ERROR</strong>: Invalid username / e-mail combination.');
-			} else {
-				do_action('retreive_password', $user_login);  // Misspelled and deprecated
-				do_action('retrieve_password', $user_login);
-
-				// Generate something random for a key...
-				$key = substr( md5( uniqid( microtime() ) ), 0, 8);
-				// Now insert the new md5 key into the db
-				$wpdb->query("UPDATE $wpdb->users SET user_activation_key = '$key' WHERE user_login = '$user_login'");
-				$message = __('Someone has asked to reset the password for the following site and username.') . "\r\n\r\n";
-				$message .= get_option('siteurl') . "\r\n\r\n";
-				$message .= sprintf(__('Username: %s'), $user_login) . "\r\n\r\n";
-				$message .= __('To reset your password visit the following address, otherwise just ignore this email and nothing will happen.') . "\r\n\r\n";
-				$message .= get_option('siteurl') . "/wp-login.php?action=rp&key=$key\r\n";
-
-				if (FALSE == wp_mail($user_email, sprintf(__('[%s] Password Reset'), get_option('blogname')), $message)) {
-					die('<p>' . __('The e-mail could not be sent.') . "<br />\n" . __('Possible reason: your host may have disabled the mail() function...') . '</p>');
-				} else {
-					wp_redirect('wp-login.php?checkemail=confirm');
-					exit();
-				}
-			}
+		$errors = retrieve_password();
+		if ( !is_wp_error($errors) ) {
+			wp_redirect('wp-login.php?checkemail=confirm');
+			exit();
 		}
 	}
 
-	if ( 'invalidkey' == $_GET['error'] ) $errors['invalidkey'] = __('Sorry, that key does not appear to be valid.');
+	if ( 'invalidkey' == $_GET['error'] ) $errors->add('invalidkey', __('Sorry, that key does not appear to be valid.'));
 
 	do_action('lost_password');
-	login_header(__('Lost Password'), '<p class="message">' . __('Please enter your username and e-mail address. You will receive a new password via e-mail.') . '</p>');
+	login_header(__('Lost Password'), '<p class="message">' . __('Please enter your username and e-mail address. You will receive a new password via e-mail.') . '</p>', $errors);
 ?>
 
 <form name="lostpasswordform" id="lostpasswordform" action="wp-login.php?action=lostpassword" method="post">
 	<p>
-		<label><?php _e('Username') ?><br />
+		<label><?php _e('Username or E-mail:') ?><br />
 		<input type="text" name="user_login" id="user_login" class="input" value="<?php echo attribute_escape(stripslashes($_POST['user_login'])); ?>" size="20" tabindex="10" /></label>
 	</p>
-	<p>
-		<label><?php _e('E-mail') ?><br />
-		<input type="text" name="user_email" id="user_email" class="input" value="<?php echo attribute_escape(stripslashes($_POST['user_email'])); ?>" size="25" tabindex="20" /></label>
-	</p>
 <?php do_action('lostpassword_form'); ?>
 	<p class="submit"><input type="submit" name="wp-submit" id="wp-submit" value="<?php _e('Get New Password &raquo;'); ?>" tabindex="100" /></p>
     <div><br clear="all" /></div>
@@ -169,92 +260,39 @@
 
 case 'resetpass' :
 case 'rp' :
-	$key = preg_replace('/[^a-z0-9]/i', '', $_GET['key']);
-	if ( empty( $key ) ) {
-		wp_redirect('wp-login.php?action=lostpassword&error=invalidkey');
-		exit();
-	}
+	$errors = reset_password($_GET['key']);
 
-	$user = $wpdb->get_row("SELECT * FROM $wpdb->users WHERE user_activation_key = '$key'");
-	if ( empty( $user ) ) {
-		wp_redirect('wp-login.php?action=lostpassword&error=invalidkey');
+	if ( ! is_wp_error($errors) ) {
+		wp_redirect('wp-login.php?checkemail=newpass');
 		exit();
 	}
 
-	do_action('password_reset');
+	wp_redirect('wp-login.php?action=lostpassword&error=invalidkey');
+	exit();
 
-	// Generate something random for a password...
-	$new_pass = wp_generate_password();
-	wp_set_password($new_pass, $user->ID);
-	$message  = sprintf(__('Username: %s'), $user->user_login) . "\r\n";
-	$message .= sprintf(__('Password: %s'), $new_pass) . "\r\n";
-	$message .= get_option('siteurl') . "/wp-login.php\r\n";
-
-	if (FALSE == wp_mail($user->user_email, sprintf(__('[%s] Your new password'), get_option('blogname')), $message)) {
-		die('<p>' . __('The e-mail could not be sent.') . "<br />\n" . __('Possible reason: your host may have disabled the mail() function...') . '</p>');
-	} else {
-		// send a copy of password change notification to the admin
-		// but check to see if it's the admin whose password we're changing, and skip this
-		if ($user->user_email != get_option('admin_email')) {
-			$message = sprintf(__('Password Lost and Changed for user: %s'), $user->user_login) . "\r\n";
-			wp_mail(get_option('admin_email'), sprintf(__('[%s] Password Lost/Changed'), get_option('blogname')), $message);
-		}
-
-		wp_redirect('wp-login.php?checkemail=newpass');
-		exit();
-	}
 break;
 
 case 'register' :
-	if ( FALSE == get_option('users_can_register') ) {
+	if ( !get_option('users_can_register') ) {
 		wp_redirect('wp-login.php?registration=disabled');
 		exit();
 	}
 
+	$user_login = '';
+	$user_email = '';
 	if ( $http_post ) {
 		require_once( ABSPATH . WPINC . '/registration.php');
 
-		$user_login = sanitize_user( $_POST['user_login'] );
-		$user_email = apply_filters( 'user_registration_email', $_POST['user_email'] );
-
-		// Check the username
-		if ( $user_login == '' )
-			$errors['user_login'] = __('<strong>ERROR</strong>: Please enter a username.');
-		elseif ( !validate_username( $user_login ) ) {
-			$errors['user_login'] = __('<strong>ERROR</strong>: This username is invalid.  Please enter a valid username.');
-			$user_login = '';
-		} elseif ( username_exists( $user_login ) )
-			$errors['user_login'] = __('<strong>ERROR</strong>: This username is already registered, please choose another one.');
-
-		// Check the e-mail address
-		if ($user_email == '') {
-			$errors['user_email'] = __('<strong>ERROR</strong>: Please type your e-mail address.');
-		} elseif ( !is_email( $user_email ) ) {
-			$errors['user_email'] = __('<strong>ERROR</strong>: The email address isn&#8217;t correct.');
-			$user_email = '';
-		} elseif ( email_exists( $user_email ) )
-			$errors['user_email'] = __('<strong>ERROR</strong>: This email is already registered, please choose another one.');
-
-		do_action('register_post');
-
-		$errors = apply_filters( 'registration_errors', $errors );
-
-		if ( empty( $errors ) ) {
-			$user_pass = wp_generate_password();
-
-			$user_id = wp_create_user( $user_login, $user_pass, $user_email );
-			if ( !$user_id )
-				$errors['registerfail'] = sprintf(__('<strong>ERROR</strong>: Couldn&#8217;t register you... please contact the <a href="mailto:%s">webmaster</a> !'), get_option('admin_email'));
-			else {
-				wp_new_user_notification($user_id, $user_pass);
-
-				wp_redirect('wp-login.php?checkemail=registered');
-				exit();
-			}
+		$user_login = $_POST['user_login'];
+		$user_email = $_POST['user_email'];
+		$errors = register_new_user($user_login, $user_email);
+		if ( !is_wp_error($errors) ) {
+			wp_redirect('wp-login.php?checkemail=registered');
+			exit();
 		}
 	}
 
-	login_header(__('Registration Form'), '<p class="message register">' . __('Register For This Site') . '</p>');
+	login_header(__('Registration Form'), '<p class="message register">' . __('Register For This Site') . '</p>', $errors);
 ?>
 
 <form name="registerform" id="registerform" action="wp-login.php?action=register" method="post">
@@ -288,69 +326,38 @@
 
 case 'login' :
 default:
-	$user_login = '';
-	$user_pass = '';
-
 	if ( !isset( $_REQUEST['redirect_to'] ) || is_user_logged_in() )
 		$redirect_to = 'wp-admin/';
 	else
 		$redirect_to = $_REQUEST['redirect_to'];
 
-	if ( $http_post ) {
-		// If cookies are disabled we can't log in even with a valid user+pass
-		if ( empty($_COOKIE[TEST_COOKIE]) )
-			$errors['test_cookie'] = __('<strong>ERROR</strong>: WordPress requires Cookies but your browser does not support them or they are blocked.');
-		
-		$user_login = $_POST['log'];
-		$user_login = sanitize_user( $user_login );
-		$user_pass  = $_POST['pwd'];
-		$rememberme = $_POST['rememberme'];
+	$user = wp_signon();
 
-		do_action_ref_array('wp_authenticate', array(&$user_login, &$user_pass));
-	} else {
-		$user = wp_validate_auth_cookie();
-		if ( !$user ) {
-			if ( empty($_GET['loggedout']) && !empty($_COOKIE[AUTH_COOKIE]) )
-				$errors['expiredsession'] = __('Your session has expired.');
-		} else {
-			$user = new WP_User($user);
-
-			// If the user can't edit posts, send them to their profile.
-			if ( !$user->has_cap('edit_posts') && ( empty( $redirect_to ) || $redirect_to == 'wp-admin/' ) )
-				$redirect_to = get_option('siteurl') . '/wp-admin/profile.php';
-			wp_safe_redirect($redirect_to);
-			exit();
-		}
-	}
-
-	if ( $user_login && $user_pass && empty( $errors ) ) {
-		$user = new WP_User(0, $user_login);
-
+	if ( !is_wp_error($user) ) {
 		// If the user can't edit posts, send them to their profile.
 		if ( !$user->has_cap('edit_posts') && ( empty( $redirect_to ) || $redirect_to == 'wp-admin/' ) )
-			$redirect_to = get_option('siteurl') . '/wp-admin/profile.php';
-
-		if ( wp_login($user_login, $user_pass) ) {
-			wp_set_auth_cookie($user->ID, $rememberme);
-			do_action('wp_login', $user_login);
-			wp_safe_redirect($redirect_to);
-			exit();
-		}
+			$redirect_to = get_option('siteurl') . '/wp-admin/profile.php';	
+		wp_safe_redirect($redirect_to);
+		exit();
 	}
+	
+	$errors = $user;
+	// Clear errors if loggedout is set.
+	if ( !empty($_GET['loggedout']) )
+		$errors = new WP_Error();
 
-	if ( $http_post && empty( $user_login ) )
-		$errors['user_login'] = __('<strong>ERROR</strong>: The username field is empty.');
-	if ( $http_post && empty( $user_pass ) )
-		$errors['user_pass'] = __('<strong>ERROR</strong>: The password field is empty.');
+	// If cookies are disabled we can't log in even with a valid user+pass
+	if ( isset($_POST['testcookie']) && empty($_COOKIE[TEST_COOKIE]) )
+		$errors->add('test_cookie', __("<strong>ERROR</strong>: Cookies are blocked or not supported by your browser. You must <a href='http://www.google.com/cookies.html'>enable cookies</a> to use WordPress."));
 
 	// Some parts of this script use the main login form to display a message
-	if		( TRUE == $_GET['loggedout'] )			$errors['loggedout']		= __('You are now logged out.');
-	elseif	( 'disabled' == $_GET['registration'] )	$errors['registerdiabled']	= __('User registration is currently not allowed.');
-	elseif	( 'confirm' == $_GET['checkemail'] )	$errors['confirm']			= __('Check your e-mail for the confirmation link.');
-	elseif	( 'newpass' == $_GET['checkemail'] )	$errors['newpass']			= __('Check your e-mail for your new password.');
-	elseif	( 'registered' == $_GET['checkemail'] )	$errors['registered']		= __('Registration complete. Please check your e-mail.');
+	if		( TRUE == $_GET['loggedout'] )			$errors->add('loggedout', __('You are now logged out.'));
+	elseif	( 'disabled' == $_GET['registration'] )	$errors->add('registerdiabled', __('User registration is currently not allowed.'));
+	elseif	( 'confirm' == $_GET['checkemail'] )	$errors->add('confirm', __('Check your e-mail for the confirmation link.'));
+	elseif	( 'newpass' == $_GET['checkemail'] )	$errors->add('newpass', __('Check your e-mail for your new password.'));
+	elseif	( 'registered' == $_GET['checkemail'] )	$errors->add('registered', __('Registration complete. Please check your e-mail.'));
 
-	login_header(__('Login'));
+	login_header(__('Login'), '', $errors);
 ?>
 
 <form name="loginform" id="loginform" action="wp-login.php" method="post">
@@ -368,6 +375,7 @@
 	<p class="submit">
 		<input type="submit" name="wp-submit" id="wp-submit" value="<?php _e('Log in'); ?> &raquo;" tabindex="100" />
 		<input type="hidden" name="redirect_to" value="<?php echo attribute_escape($redirect_to); ?>" />
+		<input type="hidden" name="testcookie" value="1" />
   <div><br clear="all" /></div>
 	</p>
 <?php else : ?>
Index: wp-includes/user.php
===================================================================
--- wp-includes/user.php	(revision 6639)
+++ wp-includes/user.php	(working copy)
@@ -1,5 +1,58 @@
 <?php
 
+function wp_signon( $credentials = '' ) {
+	if ( empty($credentials) ) {
+		if ( ! empty($_POST['log']) )
+			$credentials['user_login'] = $_POST['log'];
+		if ( ! empty($_POST['pwd']) )
+			$credentials['user_password'] = $_POST['pwd'];
+		if ( ! empty($_POST['rememberme']) )
+			$credentials['remember'] = $_POST['rememberme'];
+	}
+
+	if ( !empty($credentials['user_login']) )
+		$credentials['user_login'] = sanitize_user($credentials['user_login']);
+	if ( !empty($credentials['user_password']) )
+		$credentials['user_password'] = trim($credentials['user_password']);
+	if ( !empty($credentials['remember']) )
+		$credentials['remember'] = true;
+	else
+		$credentials['remember'] = false;
+
+	// If no credential info provided, check cookie.
+	if ( empty($credentials['user_login']) && empty($credentials['user_password']) ) {
+			$user = wp_validate_auth_cookie();
+			if ( $user ) 
+				return new WP_User($user);
+
+			if ( !empty($_COOKIE[AUTH_COOKIE]) )
+				return new WP_Error('expired_session', __('Your session has expired.'));
+
+			// If the cookie is not set, be silent.
+			return new WP_Error();
+	}
+
+	if ( empty($credentials['user_login']) || empty($credentials['user_password']) ) {
+		$error = new WP_Error();
+
+		if ( empty($credentials['user_login']) )
+			$error->add('empty_username', __('<strong>ERROR</strong>: The username field is empty.'));
+		if ( empty($credentials['user_password']) )
+			$error->add('empty_password', __('<strong>ERROR</strong>: The password field is empty.'));
+		return $error;
+	}
+
+	do_action_ref_array('wp_authenticate', array(&$credentials['user_login'], &$credentials['user_password']));
+
+	$user = wp_authenticate($credentials['user_login'], $credentials['user_password']);
+	if ( is_wp_error($user) )
+		return $user;
+
+	wp_set_auth_cookie($user->ID);
+	do_action('wp_login', $credentials['user_login']);
+	return $user;
+}
+
 function get_profile($field, $user = false) {
 	global $wpdb;
 	if ( !$user )
@@ -15,8 +68,11 @@
 
 // TODO: xmlrpc only.  Maybe move to xmlrpc.php.
 function user_pass_ok($user_login,$user_pass) {
-	$userdata = get_userdatabylogin($user_login);
-	return wp_check_password($user_pass, $userdata->user_pass);
+	$user = wp_authenticate($user_login, $user_pass);
+	if ( is_wp_error($user) )
+		return false;
+
+	return true;
 }
 
 //


Index: wp-includes/pluggable.php
===================================================================
--- wp-includes/pluggable.php	(revision 6639)
+++ wp-includes/pluggable.php	(working copy)
@@ -413,59 +413,48 @@
 }
 endif;
 
-if ( !function_exists('wp_login') ) :
 /**
  * wp_login() - Checks a users login information and logs them in if it checks out
  *
- * Use the global $error to get the reason why the login failed.
- * If the username is blank, no error will be set, so assume
- * blank username on that case.
- *
- * Plugins extending this function should also provide the global
- * $error and set what the error is, so that those checking the
- * global for why there was a failure can utilize it later.
- *
- * @since 1.2.2
- * @global string $error Error when false is returned
- *
  * @param string $username User's username
  * @param string $password User's password
- * @param bool $deprecated Not used
- * @return bool False on login failure, true on successful check
+ * @return WP_Error|WP_User WP_User object if login successful, otherwise WP_Error object.
  */
-function wp_login($username, $password, $deprecated = '') {
-	global $error;
-
+function wp_authenticate($username, $password) {
 	$username = sanitize_user($username);
 
 	if ( '' == $username )
-		return false;
+		return new WP_Error('empty_username', __('<strong>ERROR</strong>: The username field is empty.'));
 
-	if ( '' == $password ) {
-		$error = __('<strong>ERROR</strong>: The password field is empty.');
-		return false;
-	}
+	if ( '' == $password )
+		return new WP_Error('empty_password', __('<strong>ERROR</strong>: The password field is empty.'));
 
 	$user = get_userdatabylogin($username);
 
-	if ( !$user || ($user->user_login != $username) ) {
-		$error = __('<strong>ERROR</strong>: Invalid username.');
-		return false;
-	}
+	if ( !$user || ($user->user_login != $username) )
+		return new WP_Error('invalid_username', __('<strong>ERROR</strong>: Invalid username.'));
 
-	if ( !wp_check_password($password, $user->user_pass) ) {
-		$error = __('<strong>ERROR</strong>: Incorrect password.');
-		return false;
-	}
+	if ( !wp_check_password($password, $user->user_pass) )
+		return new WP_Error('incorrect_password', __('<strong>ERROR</strong>: Incorrect password.'));
 
 	// If using old md5 password, rehash.
 	if ( strlen($user->user_pass) <= 32 )
 		wp_set_password($password, $user->ID);
 
-	return true;
+	return new WP_User($user->ID);
 }
-endif;
 
+/**
+ * wp_logout() - Log the current user out
+ * @since 2.5
+ *
+ */
+function wp_logout() {
+	wp_clear_auth_cookie();
+	do_action('wp_logout');
+}
+
 if ( !function_exists('wp_validate_auth_cookie') ) :
 /**
  * wp_validate_auth_cookie() - Validates authentication cookie
@@ -1225,4 +1214,38 @@
 }
 endif;
 
+if ( !function_exists('wp_login') ) :
+/**
+ * wp_login() - Checks a users login information and logs them in if it checks out
+ *
+ * Use the global $error to get the reason why the login failed.
+ * If the username is blank, no error will be set, so assume
+ * blank username on that case.
+ *
+ * Plugins extending this function should also provide the global
+ * $error and set what the error is, so that those checking the
+ * global for why there was a failure can utilize it later.
+ *
+ * @since 1.2.2
+ * @deprecated Use wp_signin()
+ * @global string $error Error when false is returned
+ *
+ * @param string $username User's username
+ * @param string $password User's password
+ * @param bool $deprecated Not used
+ * @return bool False on login failure, true on successful check
+ */
+function wp_login($username, $password, $deprecated = '') {
+	global $error;
+
+	$user = wp_authenticate($username, $password);
+
+	if ( ! is_wp_error($user) )
+		return true;
+
+	$error = $user->get_error_message();
+	return false;
+}
+endif;
+
 ?>
@@ -413,59 +413,48 @@
 /**
- * wp_login() - Checks a users login information and logs them in if it checks out
+ * wp_authenticate() - Checks a user's login information and logs them in if it checks out
+ * @since 2.5
  *